==> apracjs-13.1.29.tgz: package/lib/database/dbHelper.js <==
﻿var MySqlLib = require("./mySql/mySqlLib");
var PgSqlLib = require("./pgSql/pgSqlLib");
var Utils = require('../common/utils.js');
var StringBuilder = require('../common/stringBuilder.js');
var Url = require('url');

class DBHelper {
  constructor(connectionSetting) {
    this.dbTypeEnum = new Object();
    this.dbTypeEnum.MySql = 0;
    this.dbTypeEnum.PgSql = 1;

    let uriInfo = Url.parse(connectionSetting, true);
    let protocol = uriInfo.protocol.toLowerCase();
    this.params = null;
    this.isEndOperation = false;

    if (protocol == 'mysql:') {
      this.dbType = this.dbTypeEnum.MySql;
      this.dbLib = new MySqlLib(connectionSetting);
    } else if (protocol == 'postgres:') {
      this.dbType = this.dbTypeEnum.PgSql;
      this.dbLib = new PgSqlLib(connectionSetting);
    } else {
      throw new Error(`当前不支持${protocol}类型的数据库`);
    }
  }

  get dbType() {
    return this._dbType;
  }

  set dbType(value) {
    this._dbType = value;
  }

  get dbLib() {
    return this._dbLib;
  }

  set dbLib(value) {
    this._dbLib = value;
  }

  get params() {
    return this._params;
  }

  set params(value) {
    this._params = value;
  }

  get isEndOperation() {
    return this._isEndOperation;
  }

  set isEndOperation(value) {
    this._isEndOperation = value;
  }

  getHasTransaction() {
    return this.dbLib.hasTransaction;
  }

  close(callback, poolConnectionIsDestroy, isEndOperation) {
    if (this.isEndOperation) { throw new Error("DBHelper已经被回收，无法执行 “close” 方法"); }
    isEndOperation = (isEndOperation == null ? false : isEndOperation);
    this.isEndOperation = isEndOperation;
    this.dbLib.close(callback, poolConnectionIsDestroy);
  }

  removeEndChar(text) {
    return text.substr(0, text.length - 1);
  }

  filterChar(text) {
    return Utils.filterChar(text);
  }

  async executeSQL(sql, params) {
    if (this.isEndOperation) { throw new Error("DBHelper已经被回收，无法执行 “executeSQL” 方法"); }
    let result = await this.dbLib.query(sql, params);
    this.params = null;
    return result;
  }

  getNamesByFieldNames(fieldNames) {
    let names = "";
    for (let i = 0; i < fieldNames.length; i++) {
      let fieldName = fieldNames[i];
      if (i > 0) names += ",";
      names += this.filterChar(fieldName); //防止Sql注入      
    }
    return names;
  }

  getNamesByParams(params) {
    let names = "";
    for (var name in params)
      names += this.filterChar(name) + ","; //防止Sql注入
    names = this.removeEndChar(names);
    return names;
  }

  getNamesByParamsToArray(params) {
    let names = new Array();
    for (var name in params)
      names.push(this.filterChar(name)); //防止Sql注入
    return names;
  }

  getValues(params, maskCode) {
    let values = "";
    for (var name in params) {
      if (maskCode == null)
        values += (this.dbLib.sqlEscape(params[name]) + ","); //防止Sql注入
      else
        values += (maskCode + this.filterChar(name) + ",");
    }
    values = this.removeEndChar(values);
    return values;
  }

  getValuesToArray(params) {
    let values = new Array();
    for (var name in params)
      values.push(this.dbLib.sqlEscape(params[name])); //防止Sql注入
    return values;
  }

  getNameValues(params, keyFieldNames) {
    let names = this.getNamesByParamsToArray(params);
    let nameValues = "";
    for (var i = 0; i < names.length; i++) {
      var name = names[i];

      if (keyFieldNames == null) {
        if (i != 0)
          nameValues += (names[i] + "=:" + names[i] + ",");
      } else {
        var exist = false;
        for (var j = 0; j < keyFieldNames.length; j++) {
          if (keyFieldNames[j] === name) {
            exist = true;
            break;
          }
        }
        if (!exist)
          nameValues += (names[i] + "=:" + names[i] + ",");
      }
    }
    nameValues = this.removeEndChar(nameValues);
    return nameValues;
  }

  getWhere(keyFieldNames) {
    let where = "";
    for (let i = 0; i < keyFieldNames.length; i++) {
      var name = keyFieldNames[i] = this.filterChar(keyFieldNames[i]); //防止Sql注入
      where += ((i != 0 ? " and " : "") + name + "=:" + name);
    }
    return where;
  }

  getWhereByParams(params) {
    let where = "";
    let i = 0;
    for (let name in params) {
      name = this.filterChar(name);
      where += ((i != 0 ? " and " : "") + name + "=:" + name);
      i += 1;
    }
    return where;
  }

  select(sql, params) {
    return this.executeSQL(sql, params);
  }

  async executeScalerSQL(sql) {
    let result = await this.select(sql, this.getParameters());
    var row = result.values[0];
    var keys = Object.keys(row);
    var key = keys[0];
    return row[key];
  }

  insert(tableName, params) {
    let names = this.getNamesByParams(params);
    let values = this.getValues(params, ':');
    let sql = `insert into ${tableName} (${names}) values(${values})`;
    return this.executeSQL(sql, params);
  }

  batchInsert(tableName, insertFieldNames, values) {
    if (values.length === 0) {
      return new Promise(function (resolver, reject) {
        global.setTimeout(function () { resolver(null); });
      });
    }

    let names = this.getNamesByFieldNames(insertFieldNames);

    let valuesstr = ``;
    for (let i = 0; i < values.length; i++) {
      let value = values[i];

      if (!value) continue;

      valuesstr += `(`;
      for (let j = 0; j < insertFieldNames.length; j++) {
        let insertFieldName = insertFieldNames[j];
        if (j > 0) valuesstr += `,`;
        let paramvalue = value[insertFieldName];
        valuesstr += this.dbLib.sqlEscape(paramvalue);
      }
      valuesstr += `),`;
    }

    valuesstr = this.removeEndChar(valuesstr);

    let sql = `insert into ${tableName} (${names}) values ${valuesstr}`;
    return this.executeSQL(sql);
  }

  batchUpdate(tableName, primaryKeyName, updateFieldNames, values) {
    if (values.length === 0) {
      return new Promise(function (resolver, reject) {
        global.setTimeout(function () { resolver(null); });
      });
    }

    let keys = [];
    let isAddKeys = false;
    let sql = `UPDATE ${tableName} set`;
    primaryKeyName = this.filterChar(primaryKeyName);//防止Sql注入
    for (let i = 0; i < updateFieldNames.length; i++) {
      let updateFieldName = updateFieldNames[i];
      let paramMap = new Map();
      for (let j = 0; j < values.length; j++) {
        let value = values[j];

        if (!value) continue;

        let paramkey = value[primaryKeyName];
        paramkey = this.dbLib.sqlEscape(paramkey);//防止Sql注入

        let paramvalue = value[updateFieldName];//防止Sql注入
        paramvalue = this.dbLib.sqlEscape(paramvalue);//防止Sql注入

        if (!isAddKeys) keys.push(paramkey);

        paramMap.set(paramkey, paramvalue);
      }

      if (!isAddKeys) {
        isAddKeys = true;
      } else {
        sql += `,`;//添加更新不同列的分割符号
      }

      sql += ` ${updateFieldName} = CASE ${primaryKeyName}`;
      paramMap.forEach((paramvalue, paramkey, map) => {
        sql += ` WHEN ${paramkey} THEN ${paramvalue}`;
      });
      sql += ` END`;
    }

    let keysStr = new StringBuilder();
    for (let i = 0, length = keys.length; i < length; i++) {
      let key = keys[i];
      if (i > 0) keysStr.append(',');
      keysStr.append(key);
    }
    sql += ` WHERE ${primaryKeyName} IN (${keysStr.toString()})`;

    return this.executeSQL(sql);
  }

  batchDelete(tableName, primaryKeyName, primaryKeyList) {
    if (primaryKeyList.length === 0) {
      return new Promise(function (resolver, reject) {
        global.setTimeout(function () { resolver(null); });
      });
    }

    primaryKeyName = this.filterChar(primaryKeyName);//防止Sql注入
    let sql = `delete from ${tableName} where ${primaryKeyName} in (:primaryKeyList)`;
    return this.executeSQL(sql, { primaryKeyList: primaryKeyList });
  }

  updateByKeyFields(tableName, keyFieldNames, params) {
    let where = this.getWhere(keyFieldNames);
    let nameValues = this.getNameValues(params, keyFieldNames);
    let sql = `update ${tableName} set ${nameValues} where ${where}`;
    return this.executeSQL(sql, params);
  }

  delete(tableName, keyField, keyValue) {
    keyField = this.filterChar(keyField);
    let sql = `delete from ${tableName} where ${keyField}=:${keyField}`;
    var obj = new Object();
    obj[keyField] = keyValue;
    return this.executeSQL(sql, obj);
  }

  deleteByKeyFields(tableName, keyFieldNames, params) {
    let where = this.getWhere(keyFieldNames);
    let sql = `delete from ${tableName} where ${where}`;
    return this.executeSQL(sql, params);
  }

  deleteByWhereParams(tableName, params) {
    let where = this.getWhereByParams(params);
    let sql = `delete from ${tableName} where ${where}`;
    return this.executeSQL(sql, params);
  }

  beginTransaction() {
    if (this.isEndOperation) { throw new Error("DBHelper已经被回收，无法执行 “beginTransaction” 方法"); }
    return this.dbLib.beginTransaction();
  }

  rollbackTransaction() {
    if (this.isEndOperation) { throw new Error("DBHelper已经被回收，无法执行 “rollbackTransaction” 方法"); }
    return this.dbLib.rollbackTransaction();
  }

  commitTransaction() {
    if (this.isEndOperation) { throw new Error("DBHelper已经被回收，无法执行 “commitTransaction” 方法"); }
    return this.dbLib.commitTransaction();
  }

  getParameters() {
    return this.params;
  }

  addParameter(name, value) {
    return this._internalAddParameter(name, value);
  }

  addLikeParameter(name, value) {
    value = this._getLikeParameterValue(value);
    return this._internalAddParameter(name, Utils.format("%{0}%", value));
  }

  static getPoolInfoList() {
    let mySqlLibPoolInfoList = MySqlLib.getPoolInfoList();
    let pgSqlLibPoolInfoList = PgSqlLib.getPoolInfoList();
    let list = mySqlLibPoolInfoList.concat(pgSqlLibPoolInfoList);
    let poolInfoHash = {};
    for (let i = 0; i < list.length; i++) {
      let item = list[i];
      if (poolInfoHash[item.name] == null) {
        poolInfoHash[item.name] = item.nowConnectionCount;
      }
      else {
        poolInfoHash[item.name] += item.nowConnectionCount;
      }
    }

    let newList = [];
    for (const name in poolInfoHash) {
      if (poolInfoHash.hasOwnProperty(name)) {
        newList.push({
          name: name,
          nowConnectionCount: poolInfoHash[name]
        });
      }
    }
    return newList;
  }

  _getLikeParameterValue(value) {
    return this.dbLib.Like(value);
  }

  _internalAddParameter(name, value) {
    var params = this.params;
    if (!params) {
      params = this.params = {};
    }

    if (params[name] != null) {
      throw new Error(`参数：${name} 重复`);
    }

    params[name] = value;
    return params;
  }

  limit(start, count) {
    return this.dbLib.limit(start, count);
  }

  toNumber(filed) {
    return this.dbLib.toNumber(filed);
  }

  IF(condition, truedata, falsedata) {
    return this.dbLib.IF(condition, truedata, falsedata);
  }

  IFNULL(expression, altValue) {
    return this.dbLib.IFNULL(expression, altValue);
  }

  Like(value) {
    return this.dbLib.Like(value);
  }

  SUBSTRING_INDEX(field, split, index) {
    return this.dbLib.Like(field, split, index);
  }

  async getTableStruct(tablename) {
    return await this.dbLib.getTableStruct(tablename);
  }
}

module.exports = DBHelper;

==> apracjs-17.1.17.tgz: package/lib/database/dbHelper.js <==
﻿var MySqlLib = require("./mySql/mySqlLib");
var PgSqlLib = require("./pgSql/pgSqlLib");
var utils = require('../common/utils.js');
var StringBuilder = require('../common/stringBuilder.js');
var Url = require('url');

class DBHelper {
  constructor(connectionSetting) {
    this.dbTypeEnum = new Object();
    this.dbTypeEnum.MySql = 0;
    this.dbTypeEnum.PgSql = 1;

    let uriInfo = Url.parse(connectionSetting, true);
    let protocol = uriInfo.protocol.toLowerCase();
    this.params = null;
    this.isEndOperation = false;

    if (protocol == 'mysql:') {
      this.dbType = this.dbTypeEnum.MySql;
      this.dbLib = new MySqlLib(connectionSetting);
    } else if (protocol == 'postgres:') {
      this.dbType = this.dbTypeEnum.PgSql;
      this.dbLib = new PgSqlLib(connectionSetting);
    } else {
      throw new Error(`当前不支持${protocol}类型的数据库`);
    }
  }

  get dbType() {
    return this._dbType;
  }

  set dbType(value) {
    this._dbType = value;
  }

  get dbLib() {
    return this._dbLib;
  }

  set dbLib(value) {
    this._dbLib = value;
  }

  get params() {
    return this._params;
  }

  set params(value) {
    this._params = value;
  }

  get isEndOperation() {
    return this._isEndOperation;
  }

  set isEndOperation(value) {
    this._isEndOperation = value;
  }

  getHasTransaction() {
    return this.dbLib.hasTransaction;
  }

  close(callback, poolConnectionIsDestroy, isEndOperation) {
    if (this.isEndOperation) { throw new Error("DBHelper已经被回收，无法执行 “close” 方法"); }
    isEndOperation = (isEndOperation == null ? false : isEndOperation);
    this.isEndOperation = isEndOperation;
    this.dbLib.close(callback, poolConnectionIsDestroy);
  }

  removeEndChar(text) {
    return text.substr(0, text.length - 1);
  }

  filterChar(text) {
    return utils.filterChar(text);
  }

  async executeSQL(sql, params) {
    if (this.isEndOperation) {
      throw new Error("DBHelper已经被回收，无法执行 “executeSQL” 方法");
    }

    if (utils.HttpContext && utils.HttpContext.hasDebugSql()) {
      var logSql = this.dbLib._queryFormat(sql, params);
      utils.addSqlLog(logSql);
    }
    let result = await this.dbLib.query(sql, params);
    this.params = null;
    return result;
  }

  getNamesByFieldNames(fieldNames) {
    let names = "";
    for (let i = 0; i < fieldNames.length; i++) {
      let fieldName = fieldNames[i];
      if (i > 0) names += ",";
      names += this.filterChar(fieldName); //防止Sql注入      
    }
    return names;
  }

  getNamesByParams(params) {
    let names = "";
    for (var name in params)
      names += this.filterChar(name) + ","; //防止Sql注入
    names = this.removeEndChar(names);
    return names;
  }

  getNamesByParamsToArray(params) {
    let names = new Array();
    for (var name in params)
      names.push(this.filterChar(name)); //防止Sql注入
    return names;
  }

  getValues(params, maskCode) {
    let values = "";
    for (var name in params) {
      if (maskCode == null)
        values += (this.dbLib.sqlEscape(params[name]) + ","); //防止Sql注入
      else
        values += (maskCode + this.filterChar(name) + ",");
    }
    values = this.removeEndChar(values);
    return values;
  }

  getValuesToArray(params) {
    let values = new Array();
    for (var name in params)
      values.push(this.dbLib.sqlEscape(params[name])); //防止Sql注入
    return values;
  }

  getNameValues(params, keyFieldNames) {
    let names = this.getNamesByParamsToArray(params);
    let nameValues = "";
    for (var i = 0; i < names.length; i++) {
      var name = names[i];

      if (keyFieldNames == null) {
        if (i != 0)
          nameValues += (names[i] + "=:" + names[i] + ",");
      } else {
        var exist = false;
        for (var j = 0; j < keyFieldNames.length; j++) {
          if (keyFieldNames[j] === name) {
            exist = true;
            break;
          }
        }
        if (!exist)
          nameValues += (names[i] + "=:" + names[i] + ",");
      }
    }
    nameValues = this.removeEndChar(nameValues);
    return nameValues;
  }

  getWhere(keyFieldNames) {
    let where = "";
    for (let i = 0; i < keyFieldNames.length; i++) {
      var name = keyFieldNames[i] = this.filterChar(keyFieldNames[i]); //防止Sql注入
      where += ((i != 0 ? " and " : "") + name + "=:" + name);
    }
    return where;
  }

  getWhereByParams(params) {
    let where = "";
    let i = 0;
    for (let name in params) {
      name = this.filterChar(name);
      where += ((i != 0 ? " and " : "") + name + "=:" + name);
      i += 1;
    }
    return where;
  }

  select(sql, params) {
    return this.executeSQL(sql, params);
  }

  async executeScalerSQL(sql) {
    let result = await this.select(sql, this.getParameters());
    var row = result.values[0];
    var keys = Object.keys(row);
    var key = keys[0];
    return row[key];
  }

  insert(tableName, params) {
    //tableName = this.filterChar(tableName); // 防止sql注入
    let names = this.getNamesByParams(params);
    let values = this.getValues(params, ':');
    let sql = `insert into ${tableName} (${names}) values(${values})`;
    return this.executeSQL(sql, params);
  }

  batchInsert(tableName, insertFieldNames, values) {
    if (values.length === 0) {
      return new Promise(function(resolver, reject) {
        global.setTimeout(function() { resolver(null); });
      });
    }

    let names = this.getNamesByFieldNames(insertFieldNames);

    let valuesstr = ``;
    for (let i = 0; i < values.length; i++) {
      let value = values[i];

      if (!value) continue;

      valuesstr += `(`;
      for (let j = 0; j < insertFieldNames.length; j++) {
        let insertFieldName = insertFieldNames[j];
        if (j > 0) valuesstr += `,`;
        let paramvalue = value[insertFieldName];
        valuesstr += this.dbLib.sqlEscape(paramvalue);
      }
      valuesstr += `),`;
    }

    valuesstr = this.removeEndChar(valuesstr);

    let sql = `insert into ${tableName} (${names}) values ${valuesstr}`;
    return this.executeSQL(sql);
  }

  batchUpdate(tableName, primaryKeyName, updateFieldNames, values) {
    if (values.length === 0) {
      return new Promise(function(resolver, reject) {
        global.setTimeout(function() { resolver(null); });
      });
    }

    let keys = [];
    let isAddKeys = false;
    let sql = `UPDATE ${tableName} set`;
    primaryKeyName = this.filterChar(primaryKeyName);//防止Sql注入
    for (let i = 0; i < updateFieldNames.length; i++) {
      let updateFieldName = updateFieldNames[i];
      let paramMap = new Map();
      for (let j = 0; j < values.length; j++) {
        let value = values[j];

        if (!value) continue;

        let paramkey = value[primaryKeyName];
        paramkey = this.dbLib.sqlEscape(paramkey);//防止Sql注入

        let paramvalue = value[updateFieldName];//防止Sql注入
        paramvalue = this.dbLib.sqlEscape(paramvalue);//防止Sql注入

        if (!isAddKeys) keys.push(paramkey);

        paramMap.set(paramkey, paramvalue);
      }

      if (!isAddKeys) {
        isAddKeys = true;
      } else {
        sql += `,`;//添加更新不同列的分割符号
      }

      sql += ` ${updateFieldName} = CASE ${primaryKeyName}`;
      paramMap.forEach((paramvalue, paramkey, map) => {
        sql += ` WHEN ${paramkey} THEN ${paramvalue}`;
      });
      sql += ` END`;
    }

    let keysStr = new StringBuilder();
    for (let i = 0, length = keys.length; i < length; i++) {
      let key = keys[i];
      if (i > 0) keysStr.append(',');
      keysStr.append(key);
    }
    sql += ` WHERE ${primaryKeyName} IN (${keysStr.toString()})`;

    return this.executeSQL(sql);
  }

  batchDelete(tableName, primaryKeyName, primaryKeyList) {
    if (primaryKeyList.length === 0) {
      return new Promise(function(resolver, reject) {
        global.setTimeout(function() { resolver(null); });
      });
    }

    primaryKeyName = this.filterChar(primaryKeyName);//防止Sql注入
    let sql = `delete from ${tableName} where ${primaryKeyName} in (:primaryKeyList)`;
    return this.executeSQL(sql, { primaryKeyList: primaryKeyList });
  }

  updateByKeyFields(tableName, keyFieldNames, params) {
    //tableName = this.filterChar(tableName);
    let where = this.getWhere(keyFieldNames);
    let nameValues = this.getNameValues(params, keyFieldNames);
    let sql = `update ${tableName} set ${nameValues} where ${where}`;
    return this.executeSQL(sql, params);
  }

  delete(tableName, keyField, keyValue) {
    //tableName = this.filterChar(tableName);
    keyField = this.filterChar(keyField);
    let sql = `delete from ${tableName} where ${keyField}=:${keyField}`;
    var params = {};
    params[keyField] = keyValue;
    return this.executeSQL(sql, params);
  }

  deleteByKeyFields(tableName, keyFieldNames, params) {
    let where = this.getWhere(keyFieldNames);
    let sql = `delete from ${tableName} where ${where}`;
    return this.executeSQL(sql, params);
  }

  deleteByWhereParams(tableName, params) {
    let where = this.getWhereByParams(params);
    let sql = `delete from ${tableName} where ${where}`;
    return this.executeSQL(sql, params);
  }

  beginTransaction() {
    if (this.isEndOperation) { throw new Error("DBHelper已经被回收，无法执行 “beginTransaction” 方法"); }
    return this.dbLib.beginTransaction();
  }

  rollbackTransaction() {
    if (this.isEndOperation) { throw new Error("DBHelper已经被回收，无法执行 “rollbackTransaction” 方法"); }
    return this.dbLib.rollbackTransaction();
  }

  commitTransaction() {
    if (this.isEndOperation) { throw new Error("DBHelper已经被回收，无法执行 “commitTransaction” 方法"); }
    return this.dbLib.commitTransaction();
  }

  getParameters() {
    return this.params;
  }

  addParameter(name, value) {
    return this._internalAddParameter(name, value);
  }

  addLikeParameter(name, value) {
    value = this._getLikeParameterValue(value);
    return this._internalAddParameter(name, utils.format("%{0}%", value));
  }

  static getPoolInfoList() {
    let mySqlLibPoolInfoList = MySqlLib.getPoolInfoList();
    let pgSqlLibPoolInfoList = PgSqlLib.getPoolInfoList();
    let list = mySqlLibPoolInfoList.concat(pgSqlLibPoolInfoList);
    let poolInfoHash = {};
    for (let i = 0; i < list.length; i++) {
      let item = list[i];
      if (poolInfoHash[item.name] == null) {
        poolInfoHash[item.name] = item.nowConnectionCount;
      }
      else {
        poolInfoHash[item.name] += item.nowConnectionCount;
      }
    }

    let newList = [];
    for (const name in poolInfoHash) {
      if (poolInfoHash.hasOwnProperty(name)) {
        newList.push({
          name: name,
          nowConnectionCount: poolInfoHash[name]
        });
      }
    }
    return newList;
  }

  _getLikeParameterValue(value) {
    return this.dbLib.Like(value);
  }

  _internalAddParameter(name, value) {
    var params = this.params;
    if (!params) {
      params = this.params = {};
    }

    if (params[name] != null) {
      throw new Error(`参数：${name} 重复`);
    }

    params[name] = value;
    return params;
  }

  limit(start, count) {
    return this.dbLib.limit(start, count);
  }

  toNumber(filed) {
    return this.dbLib.toNumber(filed);
  }

  IF(condition, truedata, falsedata) {
    return this.dbLib.IF(condition, truedata, falsedata);
  }

  IFNULL(expression, altValue) {
    return this.dbLib.IFNULL(expression, altValue);
  }

  Like(value) {
    return this.dbLib.Like(value);
  }

  SUBSTRING_INDEX(field, split, index) {
    return this.dbLib.Like(field, split, index);
  }

  async getTableStruct(tablename) {
    return await this.dbLib.getTableStruct(tablename);
  }
}

module.exports = DBHelper;
